"""
Conformance tests which do not require an internet connection or HTML parsing
libraries, and so can be run as part of the normal test suite of webcolors.

For tests which extract the relevant values, during the test run, from the online
standards documents (and so require both an internet connection and an HTML parsing
library), see the file ``definitions.py`` in this directory.

"""

# SPDX-License-Identifier: BSD-3-Clause

import unittest

import webcolors

# The mappings of color names to values below are used for conformance testing; while
# the main webcolors module makes use of alphabetized, normalized mappings to hex
# values, the mappings below are the definitions in precisely the form they take in the
# relevant standards documents (they were produced via automated extraction from the
# HTML of those documents, to avoid the possibility of human copy/paste error).
#
# Sources are:
#
# HTML 4 colors: http://www.w3.org/TR/html401/types.html#h-6.5
#
# SVG colors (which CSS 3 adopted):
# http://www.w3.org/TR/SVG/types.html#ColorKeywords
#
# Conformance of this module with the relevant standards is proven by comparing its
# output to these mappings.
HTML4_COLOR_DEFINITIONS = {
    "Black": "#000000",
    "Silver": "#C0C0C0",
    "Gray": "#808080",
    "White": "#FFFFFF",
    "Maroon": "#800000",
    "Red": "#FF0000",
    "Purple": "#800080",
    "Fuchsia": "#FF00FF",
    "Green": "#008000",
    "Lime": "#00FF00",
    "Olive": "#808000",
    "Yellow": "#FFFF00",
    "Navy": "#000080",
    "Blue": "#0000FF",
    "Teal": "#008080",
    "Aqua": "#00FFFF",
}

SVG_COLOR_DEFINITIONS = {
    "aliceblue": (240, 248, 255),
    "antiquewhite": (250, 235, 215),
    "aqua": (0, 255, 255),
    "aquamarine": (127, 255, 212),
    "azure": (240, 255, 255),
    "beige": (245, 245, 220),
    "bisque": (255, 228, 196),
    "black": (0, 0, 0),
    "blanchedalmond": (255, 235, 205),
    "blue": (0, 0, 255),
    "blueviolet": (138, 43, 226),
    "brown": (165, 42, 42),
    "burlywood": (222, 184, 135),
    "cadetblue": (95, 158, 160),
    "chartreuse": (127, 255, 0),
    "chocolate": (210, 105, 30),
    "coral": (255, 127, 80),
    "cornflowerblue": (100, 149, 237),
    "cornsilk": (255, 248, 220),
    "crimson": (220, 20, 60),
    "cyan": (0, 255, 255),
    "darkblue": (0, 0, 139),
    "darkcyan": (0, 139, 139),
    "darkgoldenrod": (184, 134, 11),
    "darkgray": (169, 169, 169),
    "darkgreen": (0, 100, 0),
    "darkgrey": (169, 169, 169),
    "darkkhaki": (189, 183, 107),
    "darkmagenta": (139, 0, 139),
    "darkolivegreen": (85, 107, 47),
    "darkorange": (255, 140, 0),
    "darkorchid": (153, 50, 204),
    "darkred": (139, 0, 0),
    "darksalmon": (233, 150, 122),
    "darkseagreen": (143, 188, 143),
    "darkslateblue": (72, 61, 139),
    "darkslategray": (47, 79, 79),
    "darkslategrey": (47, 79, 79),
    "darkturquoise": (0, 206, 209),
    "darkviolet": (148, 0, 211),
    "deeppink": (255, 20, 147),
    "deepskyblue": (0, 191, 255),
    "dimgray": (105, 105, 105),
    "dimgrey": (105, 105, 105),
    "dodgerblue": (30, 144, 255),
    "firebrick": (178, 34, 34),
    "floralwhite": (255, 250, 240),
    "forestgreen": (34, 139, 34),
    "fuchsia": (255, 0, 255),
    "gainsboro": (220, 220, 220),
    "ghostwhite": (248, 248, 255),
    "gold": (255, 215, 0),
    "goldenrod": (218, 165, 32),
    "gray": (128, 128, 128),
    "grey": (128, 128, 128),
    "green": (0, 128, 0),
    "greenyellow": (173, 255, 47),
    "honeydew": (240, 255, 240),
    "hotpink": (255, 105, 180),
    "indianred": (205, 92, 92),
    "indigo": (75, 0, 130),
    "ivory": (255, 255, 240),
    "khaki": (240, 230, 140),
    "lavender": (230, 230, 250),
    "lavenderblush": (255, 240, 245),
    "lawngreen": (124, 252, 0),
    "lemonchiffon": (255, 250, 205),
    "lightblue": (173, 216, 230),
    "lightcoral": (240, 128, 128),
    "lightcyan": (224, 255, 255),
    "lightgoldenrodyellow": (250, 250, 210),
    "lightgray": (211, 211, 211),
    "lightgreen": (144, 238, 144),
    "lightgrey": (211, 211, 211),
    "lightpink": (255, 182, 193),
    "lightsalmon": (255, 160, 122),
    "lightseagreen": (32, 178, 170),
    "lightskyblue": (135, 206, 250),
    "lightslategray": (119, 136, 153),
    "lightslategrey": (119, 136, 153),
    "lightsteelblue": (176, 196, 222),
    "lightyellow": (255, 255, 224),
    "lime": (0, 255, 0),
    "limegreen": (50, 205, 50),
    "linen": (250, 240, 230),
    "magenta": (255, 0, 255),
    "maroon": (128, 0, 0),
    "mediumaquamarine": (102, 205, 170),
    "mediumblue": (0, 0, 205),
    "mediumorchid": (186, 85, 211),
    "mediumpurple": (147, 112, 219),
    "mediumseagreen": (60, 179, 113),
    "mediumslateblue": (123, 104, 238),
    "mediumspringgreen": (0, 250, 154),
    "mediumturquoise": (72, 209, 204),
    "mediumvioletred": (199, 21, 133),
    "midnightblue": (25, 25, 112),
    "mintcream": (245, 255, 250),
    "mistyrose": (255, 228, 225),
    "moccasin": (255, 228, 181),
    "navajowhite": (255, 222, 173),
    "navy": (0, 0, 128),
    "oldlace": (253, 245, 230),
    "olive": (128, 128, 0),
    "olivedrab": (107, 142, 35),
    "orange": (255, 165, 0),
    "orangered": (255, 69, 0),
    "orchid": (218, 112, 214),
    "palegoldenrod": (238, 232, 170),
    "palegreen": (152, 251, 152),
    "paleturquoise": (175, 238, 238),
    "palevioletred": (219, 112, 147),
    "papayawhip": (255, 239, 213),
    "peachpuff": (255, 218, 185),
    "peru": (205, 133, 63),
    "pink": (255, 192, 203),
    "plum": (221, 160, 221),
    "powderblue": (176, 224, 230),
    "purple": (128, 0, 128),
    "red": (255, 0, 0),
    "rosybrown": (188, 143, 143),
    "royalblue": (65, 105, 225),
    "saddlebrown": (139, 69, 19),
    "salmon": (250, 128, 114),
    "sandybrown": (244, 164, 96),
    "seagreen": (46, 139, 87),
    "seashell": (255, 245, 238),
    "sienna": (160, 82, 45),
    "silver": (192, 192, 192),
    "skyblue": (135, 206, 235),
    "slateblue": (106, 90, 205),
    "slategray": (112, 128, 144),
    "slategrey": (112, 128, 144),
    "snow": (255, 250, 250),
    "springgreen": (0, 255, 127),
    "steelblue": (70, 130, 180),
    "tan": (210, 180, 140),
    "teal": (0, 128, 128),
    "thistle": (216, 191, 216),
    "tomato": (255, 99, 71),
    "turquoise": (64, 224, 208),
    "violet": (238, 130, 238),
    "wheat": (245, 222, 179),
    "white": (255, 255, 255),
    "whitesmoke": (245, 245, 245),
    "yellow": (255, 255, 0),
    "yellowgreen": (154, 205, 50),
}


class ConformanceTests(unittest.TestCase):
    """
    Demonstrate that this module conforms to the relevant standards documents
    governing colors on the web.

    """

    def test_html_definition_conformance(self):
        """
        Compare the results of name-to-hex conversion to the canonical hex values
        provided in the HTML 4 specification.

        """
        for color, hex_value in HTML4_COLOR_DEFINITIONS.items():
            normalized = webcolors.normalize_hex(hex_value)
            assert normalized == webcolors.name_to_hex(color)

    def test_svg_definition_conformance(self):
        """
        Compare the results of name-to-rgb-triplet conversion to the canonical
        triplet values provided in the SVG specification.

        """
        for color, triplet in SVG_COLOR_DEFINITIONS.items():
            assert triplet == webcolors.name_to_rgb(color)



"""
Tests for the HTML5 color algorithms.

"""

# SPDX-License-Identifier: BSD-3-Claus
# pylint: disable=protected-access

import unittest

import webcolors


class HTML5Tests(unittest.TestCase):
    """
    Test the functions which implement the HTML5 color algorithms.

    """

    def test_parse_simple_color(self):
        """
        Test implementation of the HTML5 simple color parsing algorithm.

        """
        test_pairs = (
            ("#ffffff", (255, 255, 255)),
            ("#000080", (0, 0, 128)),
            ("#daa520", (218, 165, 32)),
        )
        for raw, parsed in test_pairs:
            result = webcolors.html5_parse_simple_color(raw)
            assert isinstance(result, webcolors.HTML5SimpleColor)
            assert parsed == result

    def test_parse_simple_color_error(self):
        """
        Test error conditions of the HTML5 simple color parsing algorithm.

        """
        test_values = (
            "0099ccc",
            "#09c",
            "#0000",
            "#0000000",
            "#0000gg",
            "#000000".encode("ascii"),
        )
        for value in test_values:
            self.assertRaises(ValueError, webcolors.html5_parse_simple_color, value)

    def test_serialize_simple_color(self):
        """
        Test implementation of the HTML5 simple color serialization algorithm.

        """
        test_pairs = (
            ((0, 0, 0), "#000000"),
            ((0, 0, 128), "#000080"),
            ((218, 165, 32), "#daa520"),
            (webcolors.IntegerRGB(218, 165, 32), "#daa520"),
            (webcolors.HTML5SimpleColor(218, 165, 32), "#daa520"),
        )
        for raw, serialized in test_pairs:
            result = webcolors.html5_serialize_simple_color(raw)
            assert serialized == result

    def test_parse_legacy_color(self):
        """
        Test implementation of the HTML5 legacy color parsing algorithm.

        """
        # One of these is the famous "chucknorris" value. Another is a CSS 2 system
        # color. The final two are randomly-generated but believable junk
        # strings. Correct output values obtained manually.
        test_pairs = (
            ("chucknorris", (192, 0, 0)),
            ("Window", (0, 13, 0)),
            ("RE|SXLuAse", (224, 0, 224)),
            ("+=@FnnWL!Yb}5Dk", (0, 0, 176)),
            ("A" * 129, (170, 170, 170)),
        )
        for raw, parsed in test_pairs:
            result = webcolors.html5_parse_legacy_color(raw)
            assert isinstance(result, webcolors.HTML5SimpleColor)
            assert parsed == result

    def test_parse_legacy_color_names(self):
        """
        Test the HTML5 legacy color parsing of SVG/CSS3 color names.

        """
        for name in webcolors._definitions._CSS3_NAMES_TO_HEX:
            parsed = webcolors.html5_parse_legacy_color(name)
            assert parsed == webcolors.name_to_rgb(name)

    def test_parse_legacy_color_hex(self):
        """
        Test the HTML5 legacy color parsing of three- and six-digit hexadecimal
        color values.

        """
        test_values = ("#000", "#000000", "#fff", "#ffffff", "#000080")
        for value in test_values:
            parsed = webcolors.html5_parse_legacy_color(value)
            assert parsed == webcolors.hex_to_rgb(value)

    def test_parse_legacy_color_error(self):
        """
        Test error conditions of the HTML5 legacy color parsing algorithm.

        """
        test_values = ("#000000".encode("ascii"), "transparent", "")
        for value in test_values:
            self.assertRaises(ValueError, webcolors.html5_parse_legacy_color, value)



"""
Test case which exercises webcolors' conversion functions across all 16,777,216
possible hexadecimal values and all 16,777,216 possible integer rgb() triplet values.

You should not ever need to run this test; it is not part of the normal unit-test suite,
and is used only as a final check when preparing a new release of webcolors.

Because it generates each of the nearly 17 million color values multiple times, this
test case takes some time to run and consumes most or all available CPU while
running. As a consolation, it is somewhat efficient with respect to memory.

Due to the inherent imprecision of floating-point percentage values, and the fact that
the legal (with respect to the CSS standards) set of percentage rgb() triplets is
uncountably infinite, percentage rgb() triplets are not exhaustively tested here, and
the normal test suite is used to ensure correctness of the conversion functions for
those values.

The only test performed here for percentage rgb() triplets is to ensure that converting
an integer rgb() triplet to percentage and back returns the original integer values, for
consistency.

"""

# SPDX-License-Identifier: BSD-3-Clause

import unittest

import webcolors


def hex_colors():
    """
    Generator which yields all 2**24 hexadecimal color values, in order starting
    from #000000.

    """
    for i in range(16777217):
        yield f"#{i:06x}"


def int_colors():
    """
    Generator which yields all 2**24 integer rgb() triplets, in order starting from
    (0,0,0).

    """
    red_counter = tuple(range(256))
    green_counter = tuple(range(256))
    blue_counter = tuple(range(256))
    for red_value in red_counter:
        for green_value in green_counter:
            for blue_value in blue_counter:
                yield (red_value, green_value, blue_value)


class FullColorTest(unittest.TestCase):
    """
    Exercise color conversion on all testable values.

    """

    def test_full_colors(self):
        """
        Test conversion between hexadecimal and integer rgb() for
        all 2**24 possible values.

        """
        for hex_color, int_triplet in zip(hex_colors(), int_colors()):
            assert int_triplet == webcolors.hex_to_rgb(hex_color)
            assert hex_color == webcolors.rgb_to_hex(int_triplet)

    def test_triplet_conversion(self):
        """
        Test conversion between integer and percentage rgb() as fully as possible.

        Because exhaustive testing is not possible (the set of valid percentgae rgb()
        triplets is uncountably infinite), this test ensures that for each of the 2**24
        possible integer triplets, conversion to percentage and back gives the starting
        value.

        """
        for int_triplet in int_colors():
            conversion = webcolors.rgb_percent_to_rgb(
                webcolors.rgb_to_rgb_percent(int_triplet)
            )
            assert int_triplet == conversion


if __name__ == "__main__":
    unittest.main()



"""
Test the color-format conversion utilities.

"""

# SPDX-License-Identifier: BSD-3-Clause
# pylint: disable=protected-access

import unittest

import webcolors


class HexConversionTests(unittest.TestCase):
    """
    Test the functions which convert from hex color codes to other formats.

    """

    def test_hex_to_name(self):
        """
        Test conversion from hex to color name.

        """
        test_pairs = (
            ("#ffffff", "white"),
            ("#fff", "white"),
            ("#000080", "navy"),
            ("#daa520", "goldenrod"),
        )

        for hex_value, name in test_pairs:
            assert name == webcolors.hex_to_name(hex_value)

    def test_hex_to_name_unnamed(self):
        """
        A hex code which does not correspond to a named color, or does not
        correspond to a named color in the given specification, raises ValueError.

        """
        # No name in any spec.
        self.assertRaises(ValueError, webcolors.hex_to_name, "#123456")

        # This is 'goldenrod' in CSS 3 list, unnamed in HTML 4.
        self.assertRaises(
            ValueError, webcolors.hex_to_name, "#daa520", spec=webcolors.HTML4
        )

    def test_hex_to_name_specs(self):
        """
        Using one of the supported specifications succeeds; using an unsupported
        specification raises ValueError.

        """
        for supported_spec in webcolors._definitions._SUPPORTED_SPECIFICATIONS:
            result = webcolors.hex_to_name("#ffffff", spec=supported_spec)
            assert "white" == result

        for unsupported_spec in ("css1", "css4", "html5"):
            self.assertRaises(
                ValueError, webcolors.hex_to_name, "#ffffff", spec=unsupported_spec
            )

    def test_hex_to_rgb(self):
        """
        Test conversion from hex to integer RGB triplet.

        """
        test_pairs = (
            ("#fff", (255, 255, 255)),
            ("#ffffff", (255, 255, 255)),
            ("#000080", (0, 0, 128)),
        )

        for hex_value, triplet in test_pairs:
            result = webcolors.hex_to_rgb(hex_value)
            assert isinstance(result, webcolors.IntegerRGB)
            assert triplet == result

    def test_hex_to_rgb_percent(self):
        """
        Test conversion from hex to percent RGB triplet.

        """
        test_pairs = (
            ("#fff", ("100%", "100%", "100%")),
            ("#ffffff", ("100%", "100%", "100%")),
            ("#000080", ("0%", "0%", "50%")),
        )

        for hex_value, triplet in test_pairs:
            result = webcolors.hex_to_rgb_percent(hex_value)
            assert isinstance(result, webcolors.PercentRGB)
            assert triplet == result


class IntegerRGBConversionTests(unittest.TestCase):
    """
    Test the functions which convert from integer RGB triplets to other formats.

    """

    def test_rgb_to_name(self):
        """
        Test conversion from integer RGB triplet to color name.

        """
        test_pairs = (
            ((255, 255, 255), "white"),
            ((0, 0, 128), "navy"),
            ((218, 165, 32), "goldenrod"),
            (webcolors.IntegerRGB(218, 165, 32), "goldenrod"),
        )

        for triplet, name in test_pairs:
            assert name == webcolors.rgb_to_name(triplet)

    def test_rgb_to_name_unnamed(self):
        """
        An integer RGB triplet which does not correspond to a named color, or does
        not correspond to a named color in the given specification, raises ValueError.

        """
        # No name in any spec.
        self.assertRaises(ValueError, webcolors.rgb_to_name, (18, 52, 86))

        # This is 'goldenrod' in CSS 3 list, unnamed in HTML 4.
        self.assertRaises(
            ValueError, webcolors.rgb_to_name, (218, 165, 32), spec=webcolors.HTML4
        )

    def test_rgb_to_name_specs(self):
        """
        Using one of the supported specifications succeeds; an unsupported
        specification raises ValueError.

        """
        for supported_spec in webcolors._definitions._SUPPORTED_SPECIFICATIONS:
            result = webcolors.rgb_to_name((255, 255, 255), spec=supported_spec)
            assert "white" == result

        for unsupported_spec in ("css1", "css4", "html5"):
            self.assertRaises(
                ValueError,
                webcolors.rgb_to_name,
                (255, 255, 255),
                spec=unsupported_spec,
            )

    def test_rgb_to_hex(self):
        """
        Test conversion from integer RGB triplet to hex.

        """
        test_pairs = (
            ((255, 255, 255), "#ffffff"),
            ((0, 0, 128), "#000080"),
            ((218, 165, 32), "#daa520"),
        )

        for triplet, hex_value in test_pairs:
            assert hex_value == webcolors.rgb_to_hex(triplet)

    def test_rgb_to_rgb_percent(self):
        """
        Test conversion from integer RGB triplet to percent RGB triplet.

        """
        test_pairs = (
            ((255, 255, 255), ("100%", "100%", "100%")),
            ((0, 0, 128), ("0%", "0%", "50%")),
            ((218, 165, 32), ("85.49%", "64.71%", "12.5%")),
        )

        for triplet, percent_triplet in test_pairs:
            result = webcolors.rgb_to_rgb_percent(triplet)
            assert isinstance(result, webcolors.PercentRGB)
            assert percent_triplet == result


class NameConversionTests(unittest.TestCase):
    """
    Test the functions which convert from color names to other formats.

    """

    def test_name_to_hex(self):
        """
        Test correct conversion of color names to hex.

        """
        test_pairs = (
            ("white", "#ffffff"),
            ("navy", "#000080"),
            ("goldenrod", "#daa520"),
        )

        for name, hex_value in test_pairs:
            assert hex_value == webcolors.name_to_hex(name)

    def test_name_to_hex_bad_name(self):
        """
        A name which does not correspond to a color, or does not correspond to a
        color in the given specification, raises ValueError.

        """
        test_values = (
            {"name": "goldenrod", "spec": "html4"},
            {"name": "glue", "spec": "css21"},
            {"name": "breen", "spec": "css3"},
        )

        for kwarg_dict in test_values:
            self.assertRaises(ValueError, webcolors.name_to_hex, **kwarg_dict)

    def test_name_to_hex_specs(self):
        """
        Using one of the supported specifications succeeds; using an unsupported
        specification raises ValueError.

        """
        for supported_spec in webcolors._definitions._SUPPORTED_SPECIFICATIONS:
            result = webcolors.name_to_hex("white", spec=supported_spec)
            assert "#ffffff" == result

        for unsupported_spec in ("css1", "css4", "html5"):
            self.assertRaises(
                ValueError, webcolors.name_to_hex, "white", spec=unsupported_spec
            )

    def test_name_to_rgb(self):
        """
        Test conversion from color name to integer RGB triplet.

        """
        test_pairs = (
            ("white", (255, 255, 255)),
            ("navy", (0, 0, 128)),
            ("goldenrod", (218, 165, 32)),
        )

        for name, triplet in test_pairs:
            result = webcolors.name_to_rgb(name)
            assert isinstance(result, webcolors.IntegerRGB)
            assert triplet == result

    def test_name_to_rgb_percent(self):
        """
        Test conversion from color name to percent RGB triplet.

        """
        test_pairs = (
            ("white", ("100%", "100%", "100%")),
            ("navy", ("0%", "0%", "50%")),
            ("goldenrod", ("85.49%", "64.71%", "12.5%")),
        )

        for name, triplet in test_pairs:
            result = webcolors.name_to_rgb_percent(name)
            assert isinstance(result, webcolors.PercentRGB)
            assert triplet == result


class PercentRGBConversionTests(unittest.TestCase):
    """
    Test the functions which convert from percent RGB triplets to other formats.

    """

    def test_rgb_percent_to_name(self):
        """
        Test conversion from percent RGB triplet to color name.
        """
        test_pairs = (
            (("100%", "100%", "100%"), "white"),
            (("0%", "0%", "50%"), "navy"),
            (("85.49%", "64.71%", "12.5%"), "goldenrod"),
            (webcolors.PercentRGB("85.49%", "64.71%", "12.5%"), "goldenrod"),
        )

        for triplet, name in test_pairs:
            assert name == webcolors.rgb_percent_to_name(triplet)

    def test_rgb_percent_to_name_unnamed(self):
        """
        A percent RGB triplet which does not correspond to a named color, or does
        not correspond to a named color in the given specification, raises ValueError.

        """
        # No name in any spec.
        self.assertRaises(
            ValueError, webcolors.rgb_percent_to_name, ("7.06%", "20.39%", "33.73%")
        )

        # This is 'goldenrod' in CSS 3 list, unnamed in HTML 4.
        self.assertRaises(
            ValueError,
            webcolors.rgb_percent_to_name,
            ("85.49%", "64.71%", "12.5%"),
            spec=webcolors.HTML4,
        )

    def test_rgb_percent_to_name_specs(self):
        """
        Using one of the supported specifications succeeds; an unsupported
        specification raises ValueError.

        """
        for supported_spec in ("html4", "css2", "css21", "css3"):
            result = webcolors.rgb_percent_to_name(
                ("100%", "100%", "100%"), spec=supported_spec
            )
            assert "white" == result

        for unsupported_spec in ("css1", "css4", "html5"):
            self.assertRaises(
                ValueError,
                webcolors.rgb_percent_to_name,
                ("100%", "100%", "100%"),
                spec=unsupported_spec,
            )

    def test_rgb_percent_to_hex(self):
        """
        Test conversion from percent RGB triplet to hex.

        """
        test_pairs = (
            (("100%", "100%", "0%"), "#ffff00"),
            (("0%", "0%", "50%"), "#000080"),
            (("85.49%", "64.71%", "12.5%"), "#daa520"),
        )

        for triplet, hex_value in test_pairs:
            assert hex_value == webcolors.rgb_percent_to_hex(triplet)

    def test_rgb_percent_to_rgb(self):
        """
        Test conversion from percent RGB triplet to integer RGB triplet.

        """
        test_pairs = (
            (("100%", "100%", "0%"), (255, 255, 0)),
            (("0%", "0%", "50%"), (0, 0, 128)),
            (("85.49%", "64.71%", "12.5%"), (218, 165, 32)),
        )

        for triplet, int_triplet in test_pairs:
            result = webcolors.rgb_percent_to_rgb(triplet)
            assert isinstance(result, webcolors.IntegerRGB)
            assert int_triplet == result


class ConversionTests(unittest.TestCase):
    """
    Test other aspects of conversion not covered by format-specific test cases.

    """

    def test_spelling_variants(self):
        """
        When asked to name a color value that maps to either of 'gray' or 'grey' in
        CSS3, or a related color like 'darkgray'/'darkgrey', webcolors always picks
        'gray' as the spelling.

        """
        test_values = (
            (
                "#a9a9a9",
                (169, 169, 169),
                ("66.27%", "66.27%", "66.27%"),
                "darkgray",
            ),
            (
                "#2f4f4f",
                (47, 79, 79),
                ("18.43%", "30.98%", "30.98%"),
                "darkslategray",
            ),
            (
                "#696969",
                (105, 105, 105),
                ("41.18%", "41.18%", "41.18%"),
                "dimgray",
            ),
            ("#808080", (128, 128, 128), ("50%", "50%", "50%"), "gray"),
            (
                "#d3d3d3",
                (211, 211, 211),
                ("82.75%", "82.75%", "82.75%"),
                "lightgray",
            ),
            (
                "#d3d3d3",
                (211, 211, 211),
                ("82.75%", "82.75%", "82.75%"),
                "lightgray",
            ),
            (
                "#778899",
                (119, 136, 153),
                ("46.67%", "53.33%", "60.00%"),
                "lightslategray",
            ),
            ("#708090", (112, 128, 144), ("43.92%", "50%", "56.47%"), "slategray"),
        )
        for hex_value, int_tuple, percent_tuple, name in test_values:
            for converter, value in (
                (webcolors.hex_to_name, hex_value),
                (webcolors.rgb_to_name, int_tuple),
                (webcolors.rgb_percent_to_name, percent_tuple),
            ):
                assert name == converter(value, spec=webcolors.CSS3)

    def test_names_valid(self):
        """
        names() correctly returns the set of names for a given spec.

        """
        for spec, mapping in (
            (webcolors.HTML4, webcolors._definitions._HTML4_NAMES_TO_HEX),
            (webcolors.CSS2, webcolors._definitions._CSS2_NAMES_TO_HEX),
            (webcolors.CSS21, webcolors._definitions._CSS21_NAMES_TO_HEX),
            (webcolors.CSS3, webcolors._definitions._CSS3_NAMES_TO_HEX),
        ):
            with self.subTest(spec=spec):
                assert webcolors.names(spec) == list(sorted(mapping.keys()))

    def test_names_invalid(self):
        """
        names() raises ValueError when asked for an unsupported spec.

        """
        for spec in ("CSS0", "HTML12", "random"):
            with self.subTest(spec=spec):
                with self.assertRaises(ValueError):
                    webcolors.names(spec)



"""
Test accuracy of the mappings of color names and values, by extracting the
definitions of the colors from the relevant standards documents.

"""

# SPDX-License-Identifier: BSD-3-Clause
# pylint: disable=protected-access

import re
import unittest

import html5lib  # noqa: F401 pylint: disable=unused-import
import requests
from bs4 import BeautifulSoup

import webcolors


class HTML4DefinitionTests(unittest.TestCase):
    """
    Extract the names and values of the 16 defined HTML 4 colors from the online
    version of the standard, and check them against this module's definitions of them.

    """

    def setUp(self):
        """
        Fetch the HTML 4 color definitions and store them on self.html4_colors.

        """
        self.html4_colors = {}
        soup = BeautifulSoup(
            requests.get("http://www.w3.org/TR/html401/types.html", timeout=5).content,
            "html.parser",
        )
        color_table = soup.find(
            "table", attrs={"summary": "Table of color names and their sRGB values"}
        )
        for cell in color_table.findAll("td"):
            if "width" not in cell.attrs:
                color_name, color_value = cell.text.split(" = ")
                self.html4_colors[color_name] = color_value.replace('"', "").strip()

    def test_color_definitions(self):
        """
        Ensure the values in this module match those in the HTML 4 specification.

        """
        for color_name, color_value in self.html4_colors.items():
            extracted = webcolors._definitions._HTML4_NAMES_TO_HEX[color_name.lower()]
            assert color_value.lower() == extracted


class CSS21DefinitionTests(unittest.TestCase):
    """
    Extract the names and values of the 17 defined CSS 2.1 colors from the online
    version of the standard, and check them against this module's definitions of them.

    """

    def setUp(self):
        """
        Fetch the CSS2.1 color definitions and store them on self.css21_colors.

        """
        self.color_matching_re = re.compile(r"^([a-z]+) (#[a-fA-F0-9]{6})$")
        self.css21_colors = {}
        soup = BeautifulSoup(
            requests.get("http://www.w3.org/TR/CSS2/syndata.html", timeout=5).content,
            "html.parser",
        )
        color_table = soup.find("div", attrs={"id": "TanteksColorDiagram20020613"})
        for color_square in color_table.findAll("span", attrs={"class": "colorsquare"}):
            color_name, color_value = self.color_matching_re.search(
                color_square.text
            ).groups()
            self.css21_colors[color_name] = color_value

    def test_color_definitions(self):
        """
        Ensure thie values in this module match those in the CSS2.1 specification.

        """
        for color_name, color_value in self.css21_colors.items():
            extracted = webcolors._definitions._CSS21_NAMES_TO_HEX[color_name.lower()]
            assert color_value.lower() == extracted


class CSS3DefinitionTests(unittest.TestCase):
    """
    Extract the names and values of the 147 defined CSS 3 colors from the online
    version of the standard, and check them against this module's definitions of them.

    """

    def setUp(self):
        """
        Fetch the CSS3 color definitions and store them on self.css3_colors.

        """
        self.css3_colors = {}
        soup = BeautifulSoup(
            requests.get("http://www.w3.org/TR/css3-color/", timeout=5).content,
            "html5lib",
        )
        color_table = soup.findAll("table", attrs={"class": "colortable"})[1]
        color_names = [dfn.text for dfn in color_table.findAll("dfn")]
        hex_values = [
            td.text.strip()
            for td in color_table.findAll(
                "td", attrs={"class": "c", "style": "background:silver"}
            )
            if td.text.startswith("#")
        ]
        rgb_values = [
            td.text.strip()
            for td in color_table.findAll(
                "td", attrs={"class": "c", "style": "background:silver"}
            )
            if not td.text.startswith("#")
            and not td.text.startswith("&")
            and td.text.strip()
        ]
        for i, color_name in enumerate(color_names):
            self.css3_colors[color_name] = {
                "hex": hex_values[i],
                "rgb": tuple(map(int, rgb_values[i].split(","))),
            }

    def test_color_definitions(self):
        """
        Ensure the values in this module match those in the CSS3 specification.

        """
        for color_name, color_values in self.css3_colors.items():
            extracted_hex = webcolors._definitions._CSS3_NAMES_TO_HEX[
                color_name.lower()
            ]
            extracted_rgb = webcolors.name_to_rgb(color_name)
            assert color_values["hex"].lower() == extracted_hex
            assert color_values["rgb"] == extracted_rgb


if __name__ == "__main__":
    unittest.main()
